/*
 * The Pipelined Regular Expression Monitor Compiler System
 * Copyright (c) 2002 University of British Columbia
 * All Rights Reserved.
 *
 *
 * Header file for the compiler for the PREMS tool. 
 *
 * Marcio Oliveira -- 2002
 */

#ifndef COMPILE_H
#define COMPILE_H

#include <symbols.h>
#include <hash.h>

#define RECURSION_HASH_SIZE 1039
#define OR_HASH_SIZE        17 

#define PDEF  1
#define PPROD 2

#define P_TN_DEFINE           1001
#define P_TN_SIGNAL           1002
#define P_TN_INTERNAL         1003
#define P_TN_SIGNAL_ARRAY     1004
#define P_TN_INTERNAL_ARRAY   1005
#define P_TN_PROD             1006
#define P_TN_EQUALEQUAL_ARRAY 1007
#define P_TN_EQUALEQUAL       1008
#define P_TN_AND_B            1009
#define P_TN_OR_B             1010 
#define P_TN_NOT_B            1011
#define P_TN_OR_P             1012
#define P_TN_COMMA_P          1013
#define P_TN_STAR_P           1014
#define P_TN_AT_P             1015
#define P_TN_CONSTANT         1016
#define P_TN_NOTEQUAL         1017

#define P_TN_CP               1026
#define P_TN_AND_C            1027
#define P_TN_OR_C             1028
#define P_TN_NOT_C            1029
#define P_TN_TEMP             1030
#define P_TN_TRUE_C           1031
#define P_TN_FALSE_C          1032
#define P_TN_WIRE_C           1033

#define P_AN_PLUS             1050
#define P_AN_MINUS            1051
#define P_AN_CONSTANT         1052
#define P_AN_SIGNAL           1053
#define P_AN_SIGNAL_ARRAY     1054

#define P_IN_SIGNAL           1070
#define P_IN_CONSTANT         1071

typedef union {
    int       constant;
    signalPtr signal;
} actionData;

typedef actionData indexData;

struct indexNode {
    int type;
    indexData data;
};

typedef struct actionNode *actionNodePtr;
struct actionNode {
    int type;
    actionData data;
    indexNodePtr index;
    actionNodePtr left;
    actionNodePtr right;
};

typedef struct action *actionPtr;
struct action {
    actionNodePtr lhs;
    actionNodePtr rhs;
    actionPtr next;
};

struct treeNode {
    void *data;
    DdNode *dd;
    int type;
    actionPtr actionList;
    treeNodePtr left;
    treeNodePtr right;
};

typedef struct stateMachine *stateMachinePtr;
struct stateMachine {
    int index;
    int cp[2];
    char *topProduction;
    treeNodePtr parseTree;
    treeNodePtr recognizer;
    stateMachinePtr next;
}; 

void initCompile(void);
signalPtr reduceSignal(char *, int, int, int, int);
signalPtr reduceSignalList(signalPtr, signalPtr);
void reduceSignalDeclarationList(signalType, signalPtr);
void reduceDefine(char *, treeNodePtr, int);
void reduceProduction(char *, treeNodePtr, int);
void reduceStateMachine(char *);
actionPtr reduceAction(char *, indexNodePtr, actionNodePtr);
actionPtr reduceActionList(actionPtr, actionPtr);
actionNodePtr reduceActionIdent(char *);
actionNodePtr reduceActionIdentArray(char *, indexNodePtr);
actionNodePtr reduceActionConstant(int);
actionNodePtr reduceActionMinus(actionNodePtr, actionNodePtr);
actionNodePtr reduceActionPlus(actionNodePtr, actionNodePtr);
treeNodePtr reduceActionp(treeNodePtr, actionPtr);
indexNodePtr reduceIndexIdent(char *);
indexNodePtr reduceIndexConst(int);
treeNodePtr reduceVarProd(char *);
treeNodePtr reduceVar(char *);
treeNodePtr reduceVarArray(char *, indexNodePtr);
treeNodePtr reduceConstant(int);
treeNodePtr reduceEqualEqual(treeNodePtr, treeNodePtr);
treeNodePtr reduceNotEqual(treeNodePtr, treeNodePtr);
treeNodePtr reduceAndb(treeNodePtr, treeNodePtr);
treeNodePtr reduceOrb(treeNodePtr, treeNodePtr);
treeNodePtr reduceNotb(treeNodePtr);
treeNodePtr reduceOrp(treeNodePtr, treeNodePtr);
treeNodePtr reduceCommap(treeNodePtr, treeNodePtr);
treeNodePtr reduceHatp(treeNodePtr, int);
treeNodePtr reduceHatpInterval(treeNodePtr, int, int);
treeNodePtr reduceStarp(treeNodePtr);
treeNodePtr reduceAtp(treeNodePtr, treeNodePtr);
void createParseTrees(void);
DdNode** createParseTreesAux(treeNodePtr *, hashTablePtr);
DdNode** checkNonDeterminism(treeNodePtr, DdNode ***);
int isBoolExpr(treeNodePtr); 
treeNodePtr createTreeNode(void *, DdNode *, int, treeNodePtr, treeNodePtr);
void checkComparison(treeNodePtr, treeNodePtr);

#endif
