/*
 * The Production Based Monitor Compiler System
 * Copyright (c) 2002 University of British Columbia
 * All Rights Reserved.
 *
 *
 * Grammar for the PBC language
 *
 * Marcio Oliveira -- 2002
 */

%{

#include <stdlib.h>
#include <symbols.h>
#include <compile.h>

extern int srcline;
extern int defProd;
%}


%union {
    int           int_t;
    char          *string_t;
    signalType    signalType_t;
    signalPtr     signalList_t;
    treeNodePtr   treeNode_t;
    actionPtr     action_t;
    actionNodePtr actionNode_t;
    indexNodePtr  indexNode_t;
}

%token            LP RP
%token            LCB RCB  
%token            LB RB COLON DOTDOT 
%token            EQUAL
%token            COMMA ORP HAT STAR PLUS AT
%token            MINUS
%token            SEMICOLON
%token            NOTB ANDB ORB
%token            RARROW LARROW EQUALEQUAL NOTEQUAL
%token            INPUT OUTPUT IN_OUT INTERNAL
%token            DEFINE
%token            MONITOR 
%token <string_t> IDENTIFIER
%token <int_t>    CONSTANT 

%left LP RP
%left AT
%left COMMA
%left ORP
%left STAR 
%left HAT 
%left PLUS MINUS
%left LCB RCB
%left ORB ANDB
%left EQUALEQUAL NOTEQUAL
%left NOTB

%type <string_t>      identifier
%type <int_t>         constant
%type <signalType_t>  type_specifier
%type <signalList_t>  signal signal_list
%type <treeNode_t>    boolean_expression production_expression
%type <action_t>      action action_list
%type <actionNode_t>  action_expression
%type <indexNode_t>   index_expression
%type <treeNode_t>    comparison_term comparison_expression
%start spec 

%%

spec:
    signal_declaration_list 
    define_declaration_list 
    start 
    production_list {createParseTrees();};         
 
signal_declaration_list:
    signal_declaration |
    signal_declaration_list signal_declaration;

signal_declaration:
    type_specifier signal_list SEMICOLON
        {reduceSignalDeclarationList($1, $2);};

type_specifier:
    INTERNAL  {$$ = P_SIGNAL_INTERNAL;} |
    OUTPUT    {$$ = P_SIGNAL_OUTPUT;}   |
    INPUT     {$$ = P_SIGNAL_INPUT;}    |
    IN_OUT    {$$ = P_SIGNAL_INOUT;};

signal_list:
    null                      {$$ = 0;}                        |
    signal                    {$$ = reduceSignalList(0, $1);}  |
    signal COMMA signal_list  {$$ = reduceSignalList($3, $1);}; 

signal:
    identifier
        {$$ = reduceSignal($1, 0, 0, 0, srcline);}     | 
    identifier LB constant COLON constant RB
        {$$ = reduceSignal($1, $3, $5, 0, srcline);}   |
    identifier EQUAL constant 
        {$$ = reduceSignal($1, 0, 0, $3, srcline);}    |
    identifier LB constant COLON constant RB EQUAL constant
        {$$ = reduceSignal($1, $3, $5, $8, srcline);} ; 

define_declaration_list:
    null |
    define_declaration | 
    define_declaration_list define_declaration;

define_declaration:
    DEFINE {defProd = PDEF;} identifier EQUAL boolean_expression SEMICOLON
        {reduceDefine($3, $5, srcline);}; 

boolean_expression:
    comparison_expression 
        {$$ = $1;}  |
    identifier
        {$$ = reduceVarProd($1);} |
    identifier LB index_expression RB 
        {$$ = reduceVarArray($1, $3);} |
    boolean_expression ANDB boolean_expression
        {$$ = reduceAndb($1, $3);} |
    boolean_expression ORB boolean_expression
        {$$ = reduceOrb($1, $3);} |
    NOTB boolean_expression
        {$$ = reduceNotb($2);} |
    LP boolean_expression RP
        {$$ = $2;};

comparison_expression:
    comparison_term EQUALEQUAL comparison_term 
        {$$ = reduceEqualEqual($1, $3);}  |
    comparison_term NOTEQUAL comparison_term
        {$$ = reduceNotEqual($1, $3);};

comparison_term:
    identifier 
        {$$ = reduceVar($1);} |
    identifier LB index_expression RB 
        {$$ = reduceVarArray($1, $3);} |
    constant
        {$$ = reduceConstant($1);}; 

index_expression:
    identifier 
        {$$ = reduceIndexIdent($1);} |
    constant
        {$$ = reduceIndexConst($1);};

production_expression:
    boolean_expression
        {$$ = $1;} |
    production_expression ORP production_expression
        {$$ = reduceOrp($1, $3);} |
    production_expression COMMA production_expression
        {$$ = reduceCommap($1, $3);} |
    production_expression HAT  LB CONSTANT RB
        {$$ = reduceHatp($1, $4);} |
    production_expression HAT  LB CONSTANT DOTDOT CONSTANT  RB
        {$$ = reduceHatpInterval($1, $4, $6);} |
    production_expression STAR 
        {$$ = reduceStarp($1);} |
    production_expression PLUS
        {$$ = reduceCommap($1, reduceStarp($1));} |
    production_expression AT production_expression
        {$$ = reduceAtp($1, $3);} |
    LP production_expression RP 
        {$$ = $2;} |
    production_expression LCB action_list RCB
        {$$ = reduceActionp($1, $3);};           

start:
    null |
    MONITOR start_list SEMICOLON;

start_list:
    identifier   {reduceStateMachine($1);} |
    identifier COMMA start_list {reduceStateMachine($1);};

production_list:
    production |
    production production_list;

production:
    identifier {defProd = PPROD;} RARROW production_expression SEMICOLON
        {reduceProduction($1, $4, srcline);};

action_list:
    action  {$$ = reduceActionList(NULL, $1);} |
    action action_list {$$ = reduceActionList($2, $1);}; 

action:
    identifier LARROW action_expression SEMICOLON 
        {$$ = reduceAction($1, NULL, $3);} |
    identifier LB index_expression RB LARROW action_expression SEMICOLON
        {$$ = reduceAction($1, $3, $6);}; 

action_expression:
    constant
        {$$ = reduceActionConstant($1);}        |
    identifier
        {$$ = reduceActionIdent($1);}           |
    identifier LB index_expression RB 
        {$$ = reduceActionIdentArray($1, $3);}  |
    action_expression MINUS action_expression 
        {$$ = reduceActionMinus($1, $3);}       |
    action_expression PLUS action_expression
        {$$ = reduceActionPlus($1, $3);};

identifier:
    IDENTIFIER {$$ = yylval.string_t;};

constant:
    CONSTANT   {$$ = yylval.int_t;};

null: ;

%%



